from functools import wraps
import json

from jsonresponse import to_json, DEFAULT_DEBUG, CALLBACK_NAME, HttpResponse

from django.core.exceptions import PermissionDenied
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpRequest
from django.template import Template, Context
from django.utils.decorators import available_attrs
from django.views.generic.detail import SingleObjectMixin

from django.contrib.contenttypes.models import ContentType

from .base.context_processors import template_vars
from .base.models import ApiUser, AccessRight
from .base.signature import check_query

class custom_to_json(to_json):
    def __init__(self, serializer_type, error_code = 500, raises = (), **kwargs):
        super(custom_to_json, self).__init__(serializer_type, error_code)
        self.raises = raises
        if 'cls' not in kwargs:
            kwargs['cls'] = DjangoJSONEncoder
        self.kwargs = kwargs


    def api(self, func, req, *args, **kwargs):
        """
        Raises the exceptions provided by "raises" argument, else wraps the
        error message in json
        """
        if req.GET.get('raise'):
            return super(custom_to_json, self).api(func, req, *args, **kwargs)
        req.GET = req.GET.copy()
        req.GET.update({'raise': 1})
        try:
            return super(custom_to_json, self).api(func, req, *args, **kwargs)
        except self.raises:
            raise
        except Exception as e:
            response = self.err_to_response(e)
            return self.render_data(req, response, self.error_code)

to_json = custom_to_json

def get_template_vars():
    """
    returns the template vars as dict, to be used in apps code
    """
    from django.http import HttpRequest
    return template_vars(HttpRequest())

def render_template_vars(value):
    """
    renders the template vars in a string
    """
    template = Template(value)
    return template.render(Context(get_template_vars()))


def response_for_json(request, data):
    response = HttpResponse(content_type='application/json')
    json_str = json.dumps(data)
    if 'jsonpCallback' in request.GET:
        json_str = '%s(%s);' % (request.GET['jsonpCallback'], json_str)
    response.write(json_str)
    return response


def get_request_users(request):
    users = []

    users.extend(ApiUser.objects.filter(keytype=''))

    if 'orig' in request.GET and 'signature' in request.GET:
        orig = request.GET['orig']
        query = request.META['QUERY_STRING']
        signature_users = ApiUser.objects.filter(keytype='SIGN', username=orig)
        for signature_user in signature_users:
            if check_query(query, signature_user.key):
                users.append(signature_user)

    elif 'apikey' in request.GET:
        users.extend(ApiUser.objects.filter(keytype='API',
            key=request.GET['apikey']))

    elif request.META.has_key('HTTP_AUTHORIZATION'):
        (scheme, param) = request.META['HTTP_AUTHORIZATION'].split(' ',1)
        if scheme.lower() == 'basic':
            username, password = param.strip().decode('base64').split(':',1)
            users.extend(ApiUser.objects.filter(keytype='SIGN',
                    username=username, key=password))

    def ip_match(ip, match):
        if not ip:
            return True
        if ip == match:
            return True
        return False

    users = [x for x in users if ip_match(x.ipsource, request.META.get('REMOTE_ADDR'))]
    return users


def is_authorized(request, obj, perm):
    resource_type = ContentType.objects.get_for_model(obj)
    rights = AccessRight.objects.filter(resource_type=resource_type,
            resource_pk=obj.id, codename=perm)
    users = [x.apiuser for x in rights]
    return set(users).intersection(get_request_users(request))


def protected_api(perm):
    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(instance, request, *args, **kwargs):
            if not isinstance(instance, SingleObjectMixin):
                raise Exception("protected_api must be applied on a method of a class based view")
            obj = instance.get_object()
            if not is_authorized(request, obj, perm):
                raise PermissionDenied()
            return view_func(instance, request, *args, **kwargs)
        return _wrapped_view
    return decorator
