import config
import solis

import re

from django.conf import settings
from django.db import models
from django.core.cache import cache

from passerelle.datasources.models import BaseDataSource


class SolisReferential(BaseDataSource):
    base_url = models.CharField(max_length=128, blank=False,
            help_text="Solis Web Service base URL, http://.../pfi/rest/pfi/")
    referential = models.CharField(max_length=128, blank=False,
            help_text="Referential name")
    # TODO add default attributes & filters

    parameters = ('attributes', 'filters')

    def get_data(self, attributes, filters):
        # TODO handle attributes & filters params
        ref = solis.referential(referential=self.referential,
                base_url=self.base_url)
        items = ref.get('results')
        ret = []
        for i in items:
            ret.append({
                    'id': i['id'],
                    'text': i['value'],
                    })
        return sorted(ret, lambda x,y: cmp(x['text'],y['text']))

class SolisListeCommunes(BaseDataSource):
    base_url = models.CharField(max_length=128, blank=False,
            help_text="Solis Web Service base URL, http://.../pfi/rest/pfi/")
    code_dep = models.IntegerField(blank=True, null=True,
            help_text="List this department by default (code)")

    parameters = ('code_dep', 'q')

    def get_data(self, code_dep, q):
        if not code_dep:
            code_dep = self.code_dep
        if q:
            q = q.lower()
            if re.match('^\d\d', q):
                # q est le debut d'un code postal
                code_dep = q[:2]
        if not code_dep:
            return {}
        cache_key = 'solis-liste-communes-%s' % code_dep
        ref = cache.get(cache_key)
        if not ref:
            ref = solis.referential(referential='commune',
                base_url=self.base_url,
                attributes=['cp_lieu'],
                code_dep=code_dep)
            cache.set(cache_key, ref, 60*60)
        villes = ref.get('results')
        ret = []
        for v in villes:
            # {u'Attributes': {u'Attribute': {u'id': u'stdr.commune.cp_lieu',
            # u'value': 14210}}, u'Keys': {u'Key': [{u'id':
            # u'stdr.commune.code_a_com', u'value': 771}, {u'id':
            # u'stdr.commune.code_dep', u'value': 14}]}, u'id':
            # u'commune-14-771', u'value': u'NEUILLY LE MALHERBE'},
            attrs = {}
            for attr in v['Attributes']['Attribute']:
                attrs[attr['id']] = attr['value']
            text = '%0.5d %s' % (attrs['stdr.commune.cp_lieu'], v['value'].strip())
            if q and not q in text.lower():
                continue
            ret.append({'id': v['id'], 'text': text})
        return ret

class SolisListeLieux(BaseDataSource):
    base_url = models.CharField(max_length=128, blank=False,
            help_text="Solis Web Service base URL, http://.../pfi/rest/pfi/")

    parameters = ('q', 'commune', 'departement')

    def get_data(self, q, commune, departement):
        # si commune est un code solis de la forme commune-dep-com
        if commune and commune.startswith('commune-'):
            x, departement, commune = commune.split('-')
        call = solis.conciliation(config.CONCILIATION_ADRESSE,
                base_url=self.base_url,
                commune=commune, departement=departement,
                lieu='%%%s%%' % q)
        lieux = call.get('results')
        ret = []
        for l in lieux:
            # '@affinity': u'5',
            # 'CodeDepartement/@V': u'',
            # 'CodeLieu/@V': u'0110',
            # 'CodePostal/@V': u'14000',
            # 'Commune/NomCom/@V': u'CAEN',
            # 'Commune/PK/CodeCommune/@V': u'118',
            # 'NatureLieu/@Lc': u'RUE',
            # 'NomLieu/@V': u'DU BEAU SITE'
            for k,v in l.items():
                l[k] = v.strip()
            ret.append({
                'id': '%(CodeLieu/@V)s' % l,
                'text': '%(NatureLieu/@Lc)s %(NomLieu/@V)s' % l,
                'affinity': '%(@affinity)s' % l,
                })
        return ret

class SolisListeHomonymes(BaseDataSource):
    base_url = models.CharField(max_length=128, blank=False,
            help_text="Solis Web Service base URL, http://.../pfi/rest/pfi/")

    parameters = ('nom', 'prenom', 'dn')

    def get_data(self, nom, prenom, dn):
        if dn:
            dn = dn[6:]+'-'+dn[3:5]+'-'+dn[:2]
            call = solis.conciliation(config.CONCILIATION_INDIVIDU,
                    base_url=self.base_url, nom=nom, prenom=prenom, dn=dn)
        else:
            call = solis.conciliation(config.CONCILIATION_INDIVIDU_SANS_DN,
                    base_url=self.base_url, nom=nom, prenom=prenom)
        individus = call.get('results')
        ret = []
        for i in individus:
            # i = {'@affinity': u'3',
            #  'Dossier/Adresse/Commune/NomCom/@V': u'ST JULIEN EN GENEVOIS',
            #  'Dossier/Adresse/ComplementLieu/@V': u'ROUTE DE THOIRY',
            #  'Dossier/Adresse/CpLieu/@V': u'74160',
            #  'Dossier/Adresse/NatureLieu/@Lc': u'',
            #  'Dossier/Adresse/NomLieu/@V': u'.',
            #  'Dossier/Adresse/NumeroLieu/@V': u'39',
            #  'Dossier/PK/IndexDossier/@V': u'162438',
            #  'EtatCivil/DateNaissance/@V': u'1933-08-28',
            #  'EtatCivil/Nom/@V': u'DUPONT',
            #  'EtatCivil/NomJeuneFille/@V': u'BUATHIER',
            #  'EtatCivil/Prenom/@V': u'JEANNE',
            #  'PK/IndexIndividu/@V': u'208359'},
            for k,v in i.items():
                i[k] = v.strip()
            njf = i['EtatCivil/NomJeuneFille/@V']
            if njf:
                i['EtatCivil/NomJeuneFille/@V'] = u' (%s)' % njf
            if not i['EtatCivil/DateNaissance/@V']:
                i['EtatCivil/DateNaissance/@V'] = u'date de naissance inconnue'
            ret.append({
                'id': '%(PK/IndexIndividu/@V)s' % i,
                'text': ('%(EtatCivil/Nom/@V)s%(EtatCivil/NomJeuneFille/@V)s %(EtatCivil/Prenom/@V)s' + \
                        ' - %(EtatCivil/DateNaissance/@V)s' + \
                        ' - %(Dossier/Adresse/CpLieu/@V)s %(Dossier/Adresse/Commune/NomCom/@V)s') % i,
                'affinity': '%(@affinity)s' % i,
                })
        ret.sort(lambda x,y: cmp(y['affinity'],x['affinity']))
        return ret

