# -*- coding: utf-8 -*-

'''
Application emulating Concerto's webservices.
'''

try:
    import psycopg2
    import psycopg2.extras
except ImportError:
    psycopg2 = None


import hashlib
import hmac
import base64

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.dateformat import format as date_format
from django.utils.dateformat import time_format
from django.utils.translation import ugettext_lazy as _

from passerelle.datasources.models import BaseDataSource


class Concerto(BaseDataSource):
    invoice_hash_secret = models.CharField(max_length=80)
    pg_database = models.CharField(max_length=64)
    pg_user = models.CharField(max_length=64, null=True, blank=True)
    pg_password = models.CharField(max_length=64, null=True, blank=True)
    pg_host = models.CharField(max_length=64, null=True, blank=True)
    pg_port = models.CharField(max_length=64, null=True, blank=True)

    category = _('Business Process Connectors')

    class Meta:
        verbose_name = _(u'Concerto™')

    def __init__(self, *args, **kwargs):
        super(Concerto, self).__init__(*args, **kwargs)
        self.pgconn = None

    def get_absolute_url(self):
        return reverse('concerto-view', kwargs={'slug': self.slug})

    @classmethod
    def get_add_url(cls):
        return reverse('concerto-add')

    def get_edit_url(self):
        return reverse('concerto-edit', kwargs={'slug': self.slug})

    def get_delete_url(self):
        return reverse('concerto-delete', kwargs={'slug': self.slug})

    @classmethod
    def get_verbose_name(cls):
        return cls._meta.verbose_name

    @classmethod
    def get_icon_class(cls):
        return 'concerto'

    def get_connection(self, new=False):
        if psycopg2 is None:
            raise Http404
        if new and self.pgconn is not None:
            self.pgconn.close()
            self.pgconn = None
        if self.pgconn is None:
            args = {'database': self.pg_database}
            if self.pg_user:
                args['user'] = self.pg_user
                args['password'] = self.pg_password
            if self.pg_host:
                args['host'] = self.pg_host
            if self.pg_port:
                args['port'] = self.pg_port
            self.pgconn = psycopg2.connect(**args)
        return self.pgconn

    def execute_query(self, query, **kwargs):
        conn = self.get_connection()
        cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
        cursor.execute(query, kwargs)
        return cursor

    def get_hash(self, *args):
        hash = base64.b64encode(hmac.HMAC(
            key=str(self.invoice_hash_secret),
            msg='#'.join(map(lambda s: str(s), args)),
            digestmod=hashlib.sha256).digest(),
            altchars='AB')
        return hash[:8]

    def family_link(self, family_id, secret):
        query = 'SELECT COUNT(id) FROM data_famille \
                 WHERE id = %(family_id)s \
                 AND code_secret = %(secret)s'
        cursor = self.execute_query(query, family_id=family_id, secret=secret)
        exists = cursor.fetchone()[0] != 0
        query = 'SELECT link_date FROM data_liaisonnameidfamille \
                        WHERE famille_id = %(family_id)s'
        cursor = self.execute_query(query, family_id=family_id)
        links = [link[0] for link in cursor.fetchall()]
        return {'exists': exists, 'links': links}

    def family_data(self, nameid, data_filter=None):
        queries = {
            'families': "SELECT f.* FROM data_famille AS f \
                                INNER JOIN data_liaisonnameidfamille AS lp \
                                ON f.id = lp.famille_id \
                         WHERE lp.name_id = %(nameid)s",
            'kids':     "SELECT e.* FROM data_enfant AS e \
                               INNER JOIN data_liaisonnameidfamille as lp \
                               ON e.famille_id = lp.famille_id \
                         WHERE lp.name_id = %(nameid)s",
            'parents':   "SELECT f.id as famille_id, p.* FROM data_personne AS p \
                                 INNER JOIN data_liaisonparentfamille as lp \
                                 ON lp.personne_id = p.id \
                                 INNER JOIN data_famille AS f \
                                 ON f.id = lp.famille_id \
                                 INNER JOIN data_liaisonnameidfamille as lnf \
                                 ON lnf.famille_id = f.id \
                          WHERE lnf.name_id = %(nameid)s"
        }

        data = {}
        if data_filter and queries.has_key(data_filter):
            queries = {data_filter: queries[data_filter]}

        for group, query in queries.iteritems():
            cursor = self.execute_query(queries[group], nameid=nameid)
            data[group] = [getattr(self, 'format_family_%s' % group)(row) for row in cursor.fetchall()]
        return data

    def format_family_parents(self, row):
        data  = dict(row)
        data.update({'text': '{nom} {prenom}'.format(**row)})
        return data

    def format_family_kids(self, row):
        data  = dict(row)
        data.update({'text': '{nom} {prenom} - {date_naissance}'.format(**row)})
        return data

    def format_family_families(self, row):
        data  = dict(row)
        data.update({'text': u'Dossier numéro {id}'.format(**row)})
        return data

    def link_to_family(self, nameid, family_id):
        queries = (
            'DELETE FROM data_liaisonnameidfamille \
             WHERE name_id=%(nameid)s',
            'INSERT INTO data_liaisonnameidfamille(name_id, famille_id, link_date) \
             VALUES (%(nameid)s, %(famille)s, CURRENT_TIMESTAMP)'
        )
        for query in queries:
            cursor = self.execute_query(query, nameid=nameid, famille=family_id)

    def unlink_from_family(self, nameid):
        query = 'DELETE FROM data_liaisonnameidfamille \
                 WHERE name_id = %(nameid)s \
                 RETURNING *'
        cursor = self.execute_query(query, nameid=nameid)
        return cursor.fetchone()[0]

    def get_invoices(self, query, nameid):
        invoices = []
        for row in self.execute_query(query, nameid=nameid).fetchall():
            data = dict(row)
            hash = self.get_hash(data['id'], data['date_generation'], data['montant'])
            data.update({'hash': hash})
            invoices.append(data)
        return invoices

    def family_invoices(self, nameid, status):
        queries = {'all': 'SELECT invoice.*, invoice.montant as amount, \
                                  invoice.date_generation as creation_date, \
                                  (DATE(invoice.date_limite_paie) < DATE(NOW())) as expired, \
                                  invoice.date_limite_paie as expiration_date, \
                                  (invoice.paye OR \
                                  invoice.statut_tipi IS NOT DISTINCT FROM \'PAID\' OR \
                                  invoice.solde = 0) as paid, \
                                  DATE(invoice.date_reglement) as paid_date \
                           FROM data_facture AS invoice \
                                  INNER JOIN data_famille AS family \
                                  ON invoice.famille_id = family.id \
                                  INNER JOIN data_liaisonnameidfamille AS lp \
                                  ON family.id = lp.famille_id \
                           WHERE lp.name_id = %(nameid)s \
                                  AND invoice.active = \'true\' \
                           ORDER BY invoice.date_generation DESC',
                   'unpaid': 'SELECT invoice.*, \
                                     (DATE(invoice.date_limite_paie) < DATE(NOW())) as expired, \
                                     (invoice.paye OR invoice.prelevement_automatique \
                                                   OR invoice.statut_tipi \
                                                   IS NOT DISTINCT FROM \'PAID\' \
                                                   OR invoice.solde = 0) as paid \
                              FROM data_facture AS invoice \
                                   INNER JOIN data_famille AS family \
                                   ON invoice.famille_id = family.id \
                                   INNER JOIN data_liaisonnameidfamille AS lp \
                                   ON family.id = lp.famille_id \
                              WHERE lp.name_id = %(nameid)s \
                                    AND invoice.paye = \'false\' \
                                    AND invoice.active = \'true\' \
                                    AND invoice.solde <> 0 \
                                    AND invoice.statut_tipi IS DISTINCT FROM \'PAID\' \
                                    AND DATE(invoice.date_generation) <= DATE(now()) \
                                    AND DATE(invoice.date_limite_paie) >= DATE(NOW()) \
                              ORDER BY invoice.date_generation DESC'
                   }
        return self.get_invoices(queries[status], nameid)

    def get_invoice(self, invoice_id, invoice_hash):
        query = 'SELECT data_facture.id as id, \
                        data_facture.solde as amount, \
                        data_facture.date_generation as creation_date, \
                        (DATE(data_facture.date_limite_paie) < DATE(NOW())) as expired, \
                        data_facture.date_limite_paie as expiration_date, \
                        (data_facture.paye OR \
                         data_facture.statut_tipi IS NOT DISTINCT FROM \'PAID\' OR \
                         data_facture.solde = 0) as paid, \
                        DATE(data_facture.date_reponse_tipi) as tipi_paid_date, \
                        DATE(data_facture.date_reglement) as paid_date, \
                        data_facture.date_passage_perception, \
                        data_facture.prelevement_automatique, \
                        data_facture.id as refdet, \
                        data_facture.statut_tipi, \
                        \'REGIE ORLEANS\' as objet, \
                        data_facture.montant as total_amount \
        FROM data_facture \
        WHERE data_facture.id = %(invoice_id)s \
              AND data_facture.active = \'true\''
        r = self.execute_query(query, invoice_id=invoice_id).fetchone()
        if invoice_hash == self.get_hash(r['id'], r['creation_date'],
                                         r['total_amount']):
            return dict(r)

    def update_invoice(self, invoice_id, invoice_hash, status):
        get_query = 'SELECT id, date_generation, montant FROM \
                     data_facture WHERE id = %(invoice_id)s'
        update_query = 'UPDATE data_facture \
                        SET date_reponse_tipi=now(), statut_tipi=%(status) \
                        WHERE id = %(invoice_id)s \
                        RETURNING id, paye'
        invoice = self.execute_query(get_query, invoice_id=invoice_id).fetchone()
        if invoice:
            hash = self.get_hash(invoice['id'], invoice['date_generation'],
                                 invoice['montant'])
            if hash == invoice_hash:
                return self.execute_query(query, invoice_id=invoice_id,
                                          status=status).fetchone()
