from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

from model_utils.managers import InheritanceManager as ModelUtilsInheritanceManager

KEYTYPE_CHOICES = (
    ('API', 'API Key'),
    ('SIGN', 'Signature HMAC'),
)

class ApiUser(models.Model):
    username = models.CharField(max_length=50,
            verbose_name=_('Username'))
    fullname = models.CharField(max_length=50,
            verbose_name=_('Full Name'))
    description = models.TextField(blank=True,
            verbose_name=_('Description'))

    keytype = models.CharField(max_length=4, choices=KEYTYPE_CHOICES,
            blank=True, verbose_name=_('Key Type'))
    key = models.CharField(max_length=256, blank=True, verbose_name=_('Key'))

    ipsource = models.GenericIPAddressField(blank=True, null=True, unpack_ipv4=True,
            verbose_name=_('IP Address'))

    def __unicode__(self):
        return u'%s <%s>' % (self.fullname, self.username)

    def clean(self):
        if self.keytype and not self.key:
            raise ValidationError(_('Key can not be empty for type %s.') % self.keytype)


class TemplateVar(models.Model):
    name = models.CharField(max_length=64)
    value = models.CharField(max_length=128)

    def __unicode__(self):
        return u'%s - %s' % (self.name, self.value)


class InheritanceManager(ModelUtilsInheritanceManager):

    def get_slug(self, slug, request=None):
        '''
        Returns a resource by its slug
        Request based access control, if request is present
        '''
        resource = self.get_subclass(slug=slug)
        if request and not resource.is_accessible_by(request):
            raise ObjectDoesNotExist # to_json handles only this exception
        return resource

    def filter_apiuser(self, apiuser):
        '''
        Returns all resources accessible by apiuser
        '''
        return self.filter(Q(users=None) | Q(users=apiuser))


class BaseResource(models.Model):
    title = models.CharField(max_length=50)
    slug = models.SlugField()
    description = models.TextField()
    users = models.ManyToManyField(ApiUser, blank=True)

    objects = InheritanceManager()

    parameters = None

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.title

    def is_accessible_by(self, request):
        if request.user.is_superuser:
            return True
        restricted = self.users.all()
        return  not restricted or request.apiuser in restricted


class AccessRight(models.Model):
    codename = models.CharField(max_length=100, verbose_name='codename')
    resource_type = models.ForeignKey(ContentType)
    resource_pk = models.PositiveIntegerField()
    resource = generic.GenericForeignKey('resource_type', 'resource_pk')
    apiuser = models.ForeignKey(ApiUser)

    class Meta:
        permissions = (
            ('view_accessright', 'Can view access right'),
        )

    def __unicode__(self):
        return '%s (on %s <%s>) (for %s)' % (self.codename,
                self.resource_type, self.resource_pk, self.apiuser)
